#import <Foundation/NSFileHandle.h>

#import "AreaProgram.h"

@implementation AreaProgram

-(AreaProgram*)init {
	[ super init ];
	areaName = [ NSString stringWithString: @"" ];
	[ areaName retain ];
	channelPrograms = [ [ NSMutableArray alloc ] init ];
	time = 0;
	[ channelPrograms retain ];
	waveClass = nil;
	dateString = nil;
	return self;	
}

-(AreaProgram*)initWithParameters : (NSString*)name : (int) dateValue {
	[ self init ];
	[ self->areaName release ];
	self->areaName = name;
	[ self->areaName retain ];
	self->time = dateValue;
	return self;
}

-(void)dealloc {
	[ super dealloc ];
	[ channelPrograms release ];
	[ areaName release ];
	[ dateString release ];
	[ waveClass release ];
}

+(AreaProgram*)areaProgramWithFile: (NSString*)filename {
	NSFileManager* fm = [ NSFileManager defaultManager ];
	if ( [ fm fileExistsAtPath: filename ] == NO ) {
		NSLog( @"cannot find %@", filename );
		return nil;
	}
	
	NSFileHandle* fh = [ NSFileHandle fileHandleForReadingAtPath: filename ];
	NSString* contents;
	AreaProgram* area_table = [ [ AreaProgram alloc ] init ];
	
	contents = [ [ NSString alloc ] initWithData : [ fh readDataToEndOfFile ] encoding:NSUTF8StringEncoding ];
	[ fh closeFile ];
	
	//NSLog( @"loading %@", filename );
	//NSLog( @"%@", contents );
	{
		int pos = 0;
		int len = [ contents length ];
		int i;
		NSString* line;
		NSRange range;
		ProgramTable* table;
		for ( i = 0; i <= len; i++ ) {
			unichar c;
			if ( i == len ) {
				c == L'\n';
			} else {
				c = [ contents characterAtIndex: i ];
			}
			if ( c == L'\n' ) {
				range.location = pos;
				range.length = i - pos;
				pos = i + 1;
				if ( range.length > 0 ) {
					line = [ contents substringWithRange: range ];					
					/* new channel */
					if ( [ line characterAtIndex : 0 ] == L'#' ) {
						NSRange range_column;
						int j;
						int cind = 0;
						range_column.location = 1;
						range_column.length = [ line length ] - 1;
						for ( j = 1; j <= [ line length ]; j++ ) {
							unichar u;
							if ( j < [ line length ] ) {
								u = [ line characterAtIndex: j ];
							} else {
								u = L',';
							}
							if ( u == L',' ) {
								NSString* item;
								range_column.length = j - range_column.location;
								item = [ line substringWithRange: range_column ];
								switch ( cind ) {
									case 0:
										[ area_table setArea: item ];
										break;
									case 1:
										[ area_table setDate: item ];
										break;
									case 2:
										[ area_table setWave: item ];
										break;
								}
								range_column.location = j + 1;
								cind ++;
							}
						}
					} else if ( [ line characterAtIndex : 0 ] == L'*' ) {
						int j;
						int count_column = 0;
						NSRange range_column;
						NSString* channel_name = [ line substringFromIndex: 1 ];
						NSString* channel_url  = @"";
						range_column.location = 1;
						range_column.length = [ line length ] - 1;
						for ( j = 1; j < [ line length ]; j++ ) {
							if ( [ line characterAtIndex : j ] == L',' ) {
								range_column.length = j - range_column.location;
								if ( count_column == 0 ) {
									channel_name = [ line substringWithRange: range_column ];
								} else if ( count_column == 1 ) {
									channel_url = [ line substringWithRange: range_column ];
								} else {
									break;
								}
								count_column ++;
								range_column.location = j + 1;
							}
						}
						table = [ [ ProgramTable alloc ] initWithChannelNameAndURL : channel_name : channel_url ];
						[ area_table appendChannel: table ];
					} else if ( table != nil ) {
						[ table appendProgram: [ [ Program alloc ] initWithCSVString: line ] ];	
					}
				}
			}
		}
	}
	
	return area_table;
}

-(void)appendChannel : (ProgramTable*) table {
	[ channelPrograms addObject : table ];
}

-(NSArray*)getChannels {
	return channelPrograms;
}

-(int)getChannelSize {
	return [ channelPrograms count ];
}

-(ProgramTable*)getChannelAt : (int)num {
	return [ channelPrograms objectAtIndex : num ];
}

-(NSString*)getDate {
	if ( self->dateString == nil ) return @"";
	return dateString;
}

-(NSString*)getWave {
	if ( self->waveClass == nil ) return @"unknown";
	return waveClass;
}

-(void)setWave : (NSString*)wave {
	if ( self->waveClass ) {
		[ waveClass release ];
	}
	waveClass = wave;
	[ waveClass retain ];
}

-(void)setDate : (NSString*)date {
	if ( self->dateString ) {
		[ dateString release ];
	}
	dateString = date;
	[ dateString retain ];
}


-(NSString*)description {
	NSMutableString* output = [ [ NSMutableString alloc ] init ];
	int i;
	for ( i = 0; i < [ channelPrograms count ]; i++ ) {
		[ output appendString: [ [ channelPrograms objectAtIndex: i ] description ] ];
	}
	return output;
}

-(NSString*)getArea {
	return self->areaName;
}

-(void)setArea :(NSString*) area {
	[ areaName release ];
	areaName = area;
	[ areaName retain ];
}

-(NSString*) toStringProgramList: (int) timeValue {
	//NSLog( @"%s %s %d : Minutes %d", __FILE__, __func__, __LINE__, timeValue );
	NSMutableString* output = [ [ NSMutableString alloc ] init ];
	int i;
	//NSLog( @"%s %s %d", __FILE__, __func__, __LINE__ );
	for ( i = 0; i < [ channelPrograms count ]; i++ ) {
		ProgramTable* table = [ channelPrograms objectAtIndex : i ];
		NSArray* progs = [ table getProgramWithTimeRange: timeValue minutes:5 ];
		if ( [ progs count ] > 0 ) {
			[ output appendString : [ table getChannelName ] ];
			[ output appendString : @"," ];
			[ output appendString : [ [ progs objectAtIndex : 0 ] toHTML : @"<$title$>" ] ];
			[ output appendString : @"\n" ];
		}
	}
	//NSLog( @"%@", output );
	return output;
}

@end

